/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  INTERFACE DU MODULE DE TESTS AUTOMATIQUES
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include "auto.h"
#include "option.h"
#include "malloc2.h"

#define DEPTH_MAX 100

static void   _make_test(char *data, long length, int key_size,
			 int number, int depth, short ratio);
static char * _key_create(int size);
static double _run_test(char *data, long length, char *password,
			int sample_size, int depth, short ratio);
static double _key_compare(char *password1, char *password2);

static char *version="04/28/1997";
static char *option[]={"-h", "--help",
                       "-v", "--version",
                       "-f", "--infile",
                       "-d", "--depth",
                       "-p", "--path",
                       "-n", "--number",
                       "-r", "--ratio", NULL };
static int choice[]={1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7};
static char *usage[]={"[-v] [-h] [-f FILENAME] [-r RATIO] [-p PATH]",
                      "[-d DEPTH] [-n NUMBER] FIRST LAST", NULL,
                      "Repeat NUMBER tests of DEPTH parts for key's size from FIRST to LAST",
		      "",
                      "  -f --infile    choose the reference file (default is STDIN)",
                      "  -d --depth     choose the depth of the search (default is 10)",
                      "  -n --number    choose the number of tests to compute (default is 1)",
                      "  -r --ratio     choose the limit of the possible length of the key",
                      "  -p --path      choose the path for the language files (default is .)",
                      "  -h --help      display this help",
                      "  -v --version   output version information", NULL};

/*========================================================================*/

void
main(int argc, char **argv)
{
  char *name = argv[0];
  char *path = ".";
  char *file_in = NULL;
  int depth = 10;
  int number = 1;
  int first = 1;
  int last = 0;
  short ratio=20;

  while(*(++argv))
    switch(opt_argument(*argv, option, choice))
      {
      case 1:
        opt_usage(name, usage);
        exit(0);
        break;
      case 2:
        opt_version(name, version);
        exit(0);
        break;
      case 3:
        if(*(++argv))
          file_in = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 4:
        if(*(++argv))
          {
            depth = atoi(*argv);
            depth = (depth>=1 && depth<=DEPTH_MAX)?depth:10;
          }
        else
          opt_xusage(name, usage);
        break;
      case 5:
        if(*(++argv))
          path = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 6:
        if(*(++argv))
          {
            number = atoi(*argv);
            number = (number>=1 && number<=DEPTH_MAX)?number:10;
          }
        else
          opt_xusage(name, usage);
        break;
      case 7:
        if(*(++argv))
          {
            ratio = atoi(*argv);
            ratio = (ratio>=1)?ratio:15;
          }
        else
          opt_xusage(name, usage);
        break;
      default:
        if(*(argv+1))
          {
            first = atoi(*argv);
            last = atoi(*(++argv));
          }
        else
          opt_xusage(name, usage);
        break;
      }

  if(first>last)
    opt_xusage(name, usage);

  {
    int j;
    char *data = NULL;
    long length = opt_filter(file_in, &data);

    auto_install(path);
    for(j=first; j<=last; j++)
      _make_test(data, length, j, number, depth, ratio);
    auto_remove();

    free2(data);
  }

  exit(0);
}

/*==========================================================================
//  Choisit une cl de chiffrement et prpare les tests a raliser
//========================================================================*/

void
_make_test(char *data, long length, int key_size, int number,
	   int depth, short ratio)
{
  int i;
  for(i=0; i<number; i++)
    {
      int sample = key_size*50;
      double quality;

      char *password = _key_create(key_size);
      for(; (quality=_run_test(data, length, password, sample, depth, ratio))
	    <.5;
	  sample=sample-sample/10)
        printf("password %s success for %d quality %g\n", password, sample, 1-quality);
      printf("password %s stopped at %d quality %g\n", password,
	     sample, 1-quality);
      free2(password);
    }
  return;
}

/*------------------------------------------------------------------------*/

char *
_key_create(int size)
{
  char *password = (char *) malloc2(size+1);
  int i;

  for(i=0; i<size; i++)
    password[i]=(rand()%26)+'A';
  password[size]='\0';
  return password;
}

/*==========================================================================
//  Effectue la boucle de tests et retourne le resultat
//========================================================================*/

double
_run_test(char *data, long length, char *password, int sample_size,
	  int depth, short ratio)
{
  int i;
  double quality=0;

  for(i=0; i<depth; i++)
    {
      char *cipher = cipher_compute(password,
				    &data[((length-sample_size)/depth)*i],
				    sample_size, 1);
      char **newpassword = auto_decipher(cipher, sample_size, 1, ratio);
      quality+=_key_compare(password, newpassword[0]);
      free2(newpassword[0]);
      free2(newpassword[1]);
      free2(newpassword);
      free2(cipher);
    }
  return quality/((double)depth);
}

/*------------------------------------------------------------------------*/

double
_key_compare(char *password1, char *password2)
{
  double quality = 0;
  int length1 = strlen(password1);
  int length2 = strlen(password2);
  int i, j;

  if(length1<length2)
    {
      char *tmp_password = password1;
      int tmp_length = length1;
      length1 = length2;
      length2 = tmp_length;
      password1 = password2;
      password2 = tmp_password;
    }

  if((length1%length2)!=0)
    return 1;

  for(i=0; i<length1/length2; i++)
    for(j=0; j<length2; j++)
      if(password1[length2*i+j]!=password2[j])
	quality+=1/((double)length2);
  quality/=((double)i);

  return quality;
}

/*========================================================================*/
