/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  MODULE DE CHIFFREMENT DE VIGENERE
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include "cipher.h"

static char char_min = CHAR_MIN;
static char char_max = CHAR_MAX;

static int _check(char c);

/*========================================================================*/

int
_check(char c)
{
  return c >= char_min && c <= char_max;
}

/*==========================================================================
//  Redfinir l'alphabet utilise par le chiffrement
//========================================================================*/

void
cipher_define(char min, char max)
{
  if(min>max)
    {
      min = CHAR_MIN;
      max = CHAR_MAX;
    }

  char_min = min;
  char_max = max;
  return;
}

/*==========================================================================
//  Transformation d'une cl signe (+ chiffre/- dechiffre)
//========================================================================*/

char *
cipher_password(char *password, int *mode)
{
  int i;
  *mode = 1;
  switch(password[0])
    {
    case '-':
      *mode *= -1;
    case '+':
      password++;
    default:
      break;
    }

  for(i=0; _check(password[i]); i++)
    ;

  return (password[i]=='\0' && i!=0)?password:NULL;
}

/*==========================================================================
//  Chiffre (mode=1) ou dchiffre (mode=-1)
//========================================================================*/

char *
cipher_compute(char *password, char *data, long length, int mode)
{
  int i, j;
  char *result = (char *) malloc2(length);

  for(i=0, j=0; i<length; i++)
    if(_check(data[i]))
      {
        result[i] = ((data[i]-char_min)+mode*(password[j]-char_min)
                    +(char_max-char_min+1)) % (char_max-char_min+1)
                    +char_min;
        j=(password[j+1]?j+1:0);
      }
    else
      result[i] = data[i];
  return result;
}

/*========================================================================*/
