/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  INTERFACE DU MODULE DE DECRYPTAGE AUTOMATIQUE
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include "auto.h"
#include "option.h"

#define DEPTH_MAX 100

static char *version="04/08/1997";
static char *option[]={"-h", "--help",
                       "-v", "--version",
                       "-f", "--infile",
                       "-d", "--depth",
                       "-p", "--path",
                       "-r", "--ratio", NULL };
static int choice[]={1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6};
static char *usage[]={"[-v] [-h] [-f FILENAME] [-d DEPTH] [-r RATIO] [-p PATH]", NULL,
                      "Search the password of a cipher file", "",
                      "  -f --infile    choose the file to analyse (default is STDIN)",
                      "  -d --depth     choose the depth of the search (default is 1)",
                      "  -r --ratio     choose the limit of the possible length of the key",
                      "  -p --path      choose the path for the language files (default is .)",
                      "  -h --help      display this help",
                      "  -v --version   output version information", NULL};

/*========================================================================*/

void
main(int argc, char **argv)
{
  char *name = argv[0];
  char *path = ".";
  char *file_in = NULL;
  int depth = 1;
  short ratio=20;

  while(*(++argv))
    switch(opt_argument(*argv, option, choice))
      {
      case 1:
        opt_usage(name, usage);
        exit(0);
        break;
      case 2:
        opt_version(name, version);
        exit(0);
        break;
      case 3:
        if(*(++argv))
          file_in = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 4:
        if(*(++argv))
          {
            depth = atoi(*argv);
            depth = (depth>=1 && depth<=DEPTH_MAX)?depth:10;
          }
        else
          opt_xusage(name, usage);
        break;
      case 5:
        if(*(++argv))
          path = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 6:
        if(*(++argv))
          {
            ratio = atoi(*argv);
            ratio = (ratio>=1)?ratio:15;
          }
        else
          opt_xusage(name, usage);
        break;
      default:
        opt_xusage(name, usage);
        break;
      }

  {
    int i;
    char **password = NULL;
    char *data = NULL;
    long length = opt_filter(file_in, &data);

    auto_install(path);
    password = auto_decipher(data, length, depth, ratio);
    auto_remove();

    for(i=0; i<depth && password[2*i]!=NULL; i++)
      printf("%s (%s)\n", password[2*i], opt_basename(password[2*i+1]));

    free(data);
  }

  exit(0);
}

/*========================================================================*/
