/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  INTERFACE DU MODULE D'ANALYSE FREQUENTIELLE
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include "count.h"
#include "ic.h"
#include "option.h"
#include "shift.h"

static char *version="04/06/1997";
static char *option[]={"-h", "--help",
                       "-v", "--version",
                       "-s", "--step",
                       "-f", "--infile",
                       "-l", "--language",
                       "-d", "--distance", NULL };
static int choice[]={1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6};
static char *usage[]={"[-v] [-h] [[-f FILENAME] -l LANGUAGE]",
                      "[[-f FILENAME] [-d LENGTH START] [-s STEP]]", NULL,
                      "Analyse the content of a file", "",
                      "  -f --infile    choose the file to analyse (default is STDIN)",
                      "  -s --step      choose the step of the analyse (default is 1)",
                      "  -d --distance  analyse every LENGTH character from START",
                      "  -l --language  create a language file LANGUAGE",
                      "  -h --help      display this help",
                      "  -v --version   output version information", NULL};

/*========================================================================*/

void
main(int argc, char **argv)
{
  char *name = argv[0];
  char *file_in = NULL;
  char *file_out = NULL;
  int step = 1;
  int distance = 1;
  int start = 0;

  while(*(++argv))
    switch(opt_argument(*argv, option, choice))
      {
      case 1:
        opt_usage(name, usage);
        exit(0);
        break;
      case 2:
        opt_version(name, version);
        exit(0);
        break;
      case 3:
        if(*(++argv))
          {
            step = atoi(*argv);
            step = (step>0)?step:1;
          }
        else
          opt_xusage(name, usage);
        break;
      case 4:
        if(*(++argv))
          file_in = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 5:
        if(*(++argv))
          file_out = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 6:
        if(*(++argv) && *(argv+1))
          {
            distance = atoi(*(argv++));
            distance = (distance>0)?distance:1;
            start = atoi(*argv);
            start = (start>=0)?start:0;
          }
        else
          opt_xusage(name, usage);
        break;
      default:
        opt_xusage(name, usage);
      }

  {
    char *data = NULL;

    if(file_out==NULL || strcmp(file_out, "stdout")==0)
      {
        long length = opt_load(file_in, &data);
        set set = NULL;
        set = count_compute(data, length, step, distance, start);
        set_normalize(set);
        set_save(NULL, set);
        set_destroy(set);
      }
    else
      {
        long length = opt_filter(file_in, &data);
        double *table = ic_frequency(data, length, 1, 0, 2);
        unsigned char binary_table[CHAR_RANGE*CHAR_RANGE*LANGUAGE_DATA_BYTES];
        shift_double2binary(table, binary_table);
        opt_save(file_out, (char *) binary_table,
		 CHAR_RANGE*CHAR_RANGE*LANGUAGE_DATA_BYTES);
        free(table);
      }

    free(data);
  }

  exit(0);
}

/*========================================================================*/
