/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  MODULE DU CALCUL DES INDICES DE COINCIDENCE
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include "ic.h"

static double _sqr(double value);
static int    _index_shift(int index, int shift);

/*==========================================================================
//  Calcul de l'indice de coincidence d'un texte
//========================================================================*/

double
ic_coincide(char *data, long length, short cycle)
{
  short i;
  double *table;
  double result=0;

  VERIFY(data);

  for(i=0, table = ic_frequency(data, length, cycle, 0, 1);
      i<cycle;
      ic_compute(table, 1, data, length, cycle, ++i))
    result += ic_value(table, 1);

  if(cycle==0)
    RAISE(fatal_XCEPT, "cycle", NULL);
  result /= (double) cycle;

  free2(table);
  return result;
}

/*==========================================================================
//  Retourne un tableau contenant les frquences de width-lettres
//========================================================================*/

double *
ic_frequency(char *data, long length, short cycle, short start, short width)
{
  int i;
  int depth=1;
  double *table;

  VERIFY(data);

  for(i=0; i<width; i++)
    depth *= CHAR_RANGE;
  table = (double *) malloc2(depth*sizeof(*table));

  ic_compute(table, width, data, length, cycle, start);

  return table;
}

/*==========================================================================
//  Etant donne un tableau de la bonne taille, le remplit de frquences
//========================================================================*/

void
ic_compute(double *table, short width, char *data, long length,
	   short cycle, short start)
{
  long i;
  int depth=1;
  double size;

  VERIFY(table);
  VERIFY(data);

  for(i=0; i<width; i++)
    depth *= CHAR_RANGE;

  for(i=0; i<depth; i++)
    table[i]=0;

  for(i=start, size=0; i<length-width+1; i+=cycle, size++)
    {
      short j;
      int index;
      int modulo=1;
      for(j=width-1, index=0; j>=0; j--, modulo *= CHAR_RANGE)
        index += (data[i+j]-CHAR_MIN)*modulo;
      table[index]++;
    }

  if(size==0)
    RAISE(fatal_XCEPT, "start/width/length", NULL);
  for(i=0; i<depth; i++)
    table[i] /= size;

  return;
}

/*==========================================================================
//  Calcule l'indice de coincidence correspondant a un tableau de frequences
//========================================================================*/

double
ic_value(double *table, short width)
{
  int i;
  int depth=1;
  double result=0;

  VERIFY(table);

  for(i=0; i<width; i++)
    depth *= CHAR_RANGE;
  for(i=0; i<depth; i++)
    result += _sqr(table[i]);

  return result-1/((double) depth);
}

/*==========================================================================
//  Indice de coincidence mutuel a partir de 2 tableaux de frequences
//========================================================================*/

#define PSEUDO_VALUE .000001

double
ic_mutual(double *table1, double *table2, short width, int shift)
{
  int i;
  int depth=1;
  double result1=0;
  double result2=0;

  VERIFY(table1);
  VERIFY(table2);

  for(i=0; i<width; i++)
    depth *= CHAR_RANGE;
  for(i=0; i<depth; i++)
    {
      int index = _index_shift(i, shift);
      result1 += table1[i]*table2[index];
      result2 += _sqr(table1[i]-table2[index]);
    }

  return result1*(1-2*PSEUDO_VALUE)+(1-result2)*PSEUDO_VALUE+PSEUDO_VALUE;
}

/*--------------------------------------------------------------------------
//  Calcule la position de shift-lettres apres decalage alphabetique
//------------------------------------------------------------------------*/

int
_index_shift(int index, int shift)
{
  int newindex=0;
  int factor;

  for(factor=1; index || factor==1; index/=CHAR_RANGE, factor*=CHAR_RANGE)
    newindex += ((index+shift)%CHAR_RANGE)*factor;

  return newindex;
}

/*------------------------------------------------------------------------*/

double
_sqr(double value)
{
  return value*value;
}

/*========================================================================*/
