/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  MODULE D'UTILITAIRES D'INTERFACAGE AVEC LA LIGNE DE COMMANDES
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>

#if !(__dest_os) || __dest_os != __be_os
#include <unistd.h>
#endif

#include "option.h"

/*========================================================================*/

int
opt_argument(char *word, char **option, int *choice)
{
  for(; *option; option++, choice++)
    if(strcmp(*option, word)==0)
      return *choice;
  return 0;
}

/*==========================================================================
//  Gestion de USAGE
//========================================================================*/

void
opt_usage(char *name, char **usage)
{
  char *basename = opt_basename(name);
  char *blank = (char *) malloc2(strlen(basename));
  int i;
  for(i=0; basename[i]!='\0'; i++)
    blank[i]=' ';
  fprintf(stderr, "\nUsage: \033[1;5;30m%s\033[m %s\n", basename, *usage);
  while(*(++usage))
    fprintf(stderr, "       %s %s\n", blank, *usage);
  fprintf(stderr, "\n");
  while(*(++usage))
    fprintf(stderr, "%s\n", *usage);
  fprintf(stderr, "\n");
  return;
}

/*------------------------------------------------------------------------*/

void
opt_xusage(char *name, char **usage)
{
  opt_usage(name, usage);
  exit(1);
}

/*------------------------------------------------------------------------*/

char *
opt_basename(char *name)
{
  char *basename = name;
  while(*basename)
    basename++;
  while(*basename!='/' && basename>=name)
    basename--;
  return ++basename;
}

/*==========================================================================
//  Gestion de VERSION
//========================================================================*/

void
opt_version(char *name, char *version)
{
  fprintf(stderr, "%s, Copyright Lompo/de Maillard/Montiton/Morreeuw %s\n",
	  opt_basename(name), version);
  return;
}

/*==========================================================================
//  Lecture et filtrage des caractres nuls d'un nom de fichier
//========================================================================*/


long
opt_load(char *filename, char **data)
{
  int fd;
  long i, j;
  long length;

  *data = NULL;

  if((fd=io_open(filename, O_RDONLY))==-1)
    RAISE(io_XCEPT, filename==NULL?"stdin":filename, NULL);

  length = io_read(fd, data);

  if(fd!=STDIN_FILENO)
    if(close(fd)==-1)
      RAISE(io_XCEPT, filename, NULL);

  for(i=0, j=0; i<length; i++)
    if((*data)[i]!='\0')
      (*data)[j++]=(*data)[i];
  length = j;

  if(length<=0)
    RAISE(io_XCEPT, filename, "No data found");
  return length;
}

/*==========================================================================
//  Lecture et filtrage en fonction de l'alphabet
//========================================================================*/

long
opt_filter(char *filename, char **data)
{
  long i, j;
  long length = opt_load(filename, data);

  for(i=0, j=0; i<length; i++)
    if((*data)[i]>=CHAR_MIN && (*data)[i]<=CHAR_MAX)
      (*data)[j++]=(*data)[i];

  if(j==0)
    RAISE(io_XCEPT, filename, "No data found");
  return j;
}

/*========================================================================*/

void
opt_save(char *filename, char *data, long length)
{
  int fd;
  long newlength;

  if((fd=io_open(filename, O_WRONLY|O_CREAT|O_TRUNC))==-1)
    RAISE(io_XCEPT, filename, NULL);

  newlength = io_write(fd, data, length);

  if(fd!=STDOUT_FILENO)
    {
      if(close(fd)==-1)
        RAISE(io_XCEPT, filename, NULL);
      if(newlength!=length)
        RAISE(io_XCEPT, filename==NULL?"stdout":filename, NULL);
    }

  return;
}

/*========================================================================*/
