/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  MODULE DE RECHERCHE DE LA LONGUEUR DE LA CLE
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include "search.h"

#define DEFAULT_MAX_SIZE 128

static set      _compute(char *data, long length, int step);
static void     _allpgcd(set set);
static long     _pgcd(long value1, long value2);
static result_p _order(set set, int depth);

/*==========================================================================
//  Recherche de la longueur de la cl
//========================================================================*/

result_p
search_key_length(char *data, long length, short step, int buffer, int depth)
{
  long i;
  result_p result;
  set set_result = NULL;

  VERIFY(data);

  for(i=0; i<length; i+=buffer)
    {
      long size_tmp = ((i+buffer)<length)?buffer:length-i;
      set set_tmp = _compute(&data[i], size_tmp, step);
      set_result = set_addition(set_result, set_tmp);
      set_destroy(set_tmp);
    }

  _allpgcd(set_result);
  result = _order(set_result, depth);
  set_destroy(set_result);

  result_clean(result);
  result_normalize(result);

  return result;
}

/*==========================================================================
//  Recherche des rptitions
//========================================================================*/

set
_compute(char *data, long length, int step)
{
  long i, j;
  set set_string = set_create(SET_STRING);
  set set_double = set_create(SET_DOUBLE);
  char key_tmp[DEFAULT_MAX_SIZE+1];
  char *key = NULL;

  if(step>DEFAULT_MAX_SIZE)
    key = (char *) malloc2((step+1)*sizeof(*key));
  else
    key = key_tmp;

  key[step]='\0';

  VERIFY(data);

  for(i=0; i<length-step+1; i++)
    {
      double position;

      for(j=0; j<step; j++)
        key[j]=data[i+j];

      position = set_member(set_string, (void *) key);

      if(position)
        {
          double distance = i+1-position;
          set_insert(set_double, (void *) &distance);
          set_select(set_string, (void *) key, distance);
        }
      else
        set_select(set_string, (void *) key, i+1);
    }

  if(key!=key_tmp)
    free2(key);
  set_destroy(set_string);
  return set_double;
}

/*==========================================================================
//  Insertion du pgcd si diffrent des valeurs
//========================================================================*/

void
_allpgcd(set set)
{
  void *key1 = NULL;
  void *key2 = NULL;

  VERIFY(set);

  while((key2 = (key1 = set_forward(set, key1))))
    while((key2 = set_forward(set, key2)))
      {
        double value = _pgcd(*(double *) key1, *(double *) key2);
        set_select(set, (void *) &value, set_member(set, key1));
      }

  return;
}

/*------------------------------------------------------------------------*/

long
_pgcd(long value1, long value2)
{
  long a = value1;
  long b = value2;
  long r = b;

  if(a<b)
    {
      r = a;
      a = b;
      b = r;
    }

  while(r)
    {
      r = a%b;
      a = b;
      b = r;
    }

  return a;  
}

/*==========================================================================
//  Cration de la table de rsultats  partir de la structure des distances
//========================================================================*/

result_p
_order(set set, int depth)
{
  int entry;
  void *key = NULL;
  result_p result = result_create(depth, 1);

  VERIFY(set);

  entry = set_entry(set);

  while((key = set_forward(set, key)))
    {
      short *offset = result_newoffset(result);
      double *quality = result_newquality(result);
      offset[0] = (short) *((double *) key);
      quality[0] = set_member(set, key);
      result_insert(result);
    }

  return result;
}

/*========================================================================*/
