/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  MODULE DE TRI DYNAMIQUE PAR SKIPLIST A DOUBLE ACCES AUX DONNEES
//
//========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#if !(__dest_os) || __dest_os != __be_os
#include <unistd.h>
#endif

#include "set.h"

/*========================================================================*/

struct set
{
  int type;
  long entry;
  double size;
  double step;
  dsl dsl;
};

/*------------------------------------------------------------------------*/

static int _scompare(char *key1, char *key2);
static int _lcompare(double *key1, double *key2);

/*========================================================================*/

int
_scompare(char *key1, char *key2)
{
  for(; *key1==*key2 && *key1; key1++, key2++)
    ;
  return (((unsigned char ) *key1)<((unsigned char)*key2))?1:0;
}

/*------------------------------------------------------------------------*/

int
_lcompare(double *key1, double *key2)
{
  return *key1>*key2;
}

/*========================================================================*/

set
set_create(int type)
{
  set set = (struct set *) malloc(sizeof(*set));
  set->type = type;
  set->entry = 0;
  set->size = 0;
  set->step = 1;

  switch(type)
    {
    case SET_DOUBLE:
      set->dsl = dsl_create((int (*)(void *, void *))_lcompare);
      break;
    case SET_STRING:
      set->dsl = dsl_create((int (*)(void *, void *))_scompare);
      break;
    default:
      break;
    }

  return set;
}

/*========================================================================*/

double
set_member(set set, void *key)
{
  if(dsl_member(set->dsl, key))
    return *(double *) dsl_find(set->dsl, key);
  else
    return 0;
}

/*========================================================================*/

double
set_insert(set set, void *key)
{
  return set_select(set, key, set->step);
}

/*==========================================================================
//  Incrmente la valeur actuelle pour key de value
//========================================================================*/

double
set_select(set set, void *key, double value)
{
  set->size += value;

  if(dsl_member(set->dsl, key))
    return (*(double *) dsl_find(set->dsl, key)) += value;
  else
    {
      void *key_new = NULL;
      double *value_new = (double *) malloc(sizeof(*value_new));

      *value_new = value;

      switch(set->type)
        {
        case SET_DOUBLE:
          key_new = (void *) malloc(sizeof(double));
          *((double *) key_new) = *((double *) key);
          break;
        case SET_STRING:
          key_new = (void *) strdup((char *) key);
          break;
        default:
          break;
        }

      dsl_insert(set->dsl, key_new, (void *) value_new);
      set->entry++;
      return set->step;
    }
}

/*==========================================================================
//  Nombre d'lements insrs dans l'ensemble
//========================================================================*/

long
set_entry(set set)
{
  return set->entry;
}

/*========================================================================*/

set
set_addition(set destination, set origin)
{
  void *key = NULL;

  if(!destination)
    destination = set_create(origin->type);
  if(!origin)
    return destination;
  if(origin->type!=destination->type)
    return NULL;

  if(origin->step!=destination->step)
    {
      set_normalize(origin);
      set_normalize(destination);
    }

  while( (key=dsl_next(origin->dsl, key)) )
    set_select(destination, key, set_member(origin, key));

  return destination;
}

/*========================================================================*/

void
set_normalize(set set)
{
  void *key = NULL;

  if(set->size==1)
    return;

  while( (key=dsl_next(set->dsl, key)) )
    (*(double *) dsl_find(set->dsl, key)) /= set->size;

  set->step /= set->size;
  set->size = 1;
  return;
}

/*==========================================================================
//  Permet d'accder aux donnes de manire squentielle
//========================================================================*/

void *
set_forward(set set, void *key)
{
  return dsl_next(set->dsl, key);
}

/*==========================================================================
//  Gre les caractres spciaux pour avoir un bel affichage...
//========================================================================*/

void
set_save(char *filename, set set)
{
  int i;
  int fd = io_open(filename, O_WRONLY|O_CREAT|O_TRUNC);
  FILE *file = fdopen(dup(fd), "at");
  void *key = NULL;

  while( (key=dsl_next(set->dsl, key)) )
    switch(set->type)
      {
      case SET_DOUBLE:
        fprintf(file, "%g %.15g\n", *(double *) key,
                      *(double *) dsl_find(set->dsl, key));
        break;
      case SET_STRING:
        for(i=0; ((char *) key)[i]!='\0' || i==0; i++)
          if(((char *) key)[i]!='\\' &&
             ((char *) key)[i]>=' ' && ((char *) key)[i]<127)
            fprintf(file, "%c", ((char *) key)[i]);
          else
            fprintf(file, "\\%.2x", (unsigned char) ((char *) key)[i]);
        fprintf(file, " %.15g\n", *(double *) dsl_find(set->dsl, key));
        break;
      default:
        break;
      }

  fclose(file);
  if(fd!=STDOUT_FILENO)
    close(fd);
  return;
}

/*========================================================================*/

void
set_destroy(set set)
{
  void *key;

  for(key=dsl_next(set->dsl, NULL); key;)
    {
      void *key_next = dsl_next(set->dsl, key);
      free(dsl_find(set->dsl, key));
      dsl_delete(set->dsl, key);
      free(key);
      key=key_next;
    }
  dsl_destroy(set->dsl);
  free(set);
  return;
}

/*========================================================================*/
