/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  MODULE DE GESTION DES EXCEPTIONS
//
//  Ce module est une version amliore du module d'exceptions tudi en
//  TD de matrise 1996-1997. La premire modification consiste  prendre
//  en considration les erreurs systmes, la seconde  implanter une
//  pseudo exception permettant de faire suivre une exception intercepte.
//
//========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include "xcept.h"

typedef struct _context *context;

struct _context
{
  jmp_buf context_buffer;
  context previous;
  xcept   header;
};

context top = NULL;

DEFINE_XCEPT(fatal_XCEPT);
xcept last_XCEPT=NULL;

static char *       last_message = NULL;
static char *       last_m_error = NULL;
static char *       filename = NULL;
static unsigned int fileline = 0;

/*------------------------------------------------------------------------*/

void xcept_push(xcept exception)
{
  context new_context = (context) malloc(sizeof(*new_context));

  last_message=NULL;
  last_m_error=NULL;
  filename=NULL;
  fileline=0;

  if(!new_context)
    RAISE(fatal_XCEPT, "exception", "Fatal low memory condition");

  new_context->previous = top;
  new_context->header = exception;
  top = new_context;
}

/*------------------------------------------------------------------------*/

void xcept_pop(void)
{
  context tmp_context = top->previous;

  free(top);
  top = tmp_context;
}

/*------------------------------------------------------------------------*/

int xcept_search(xcept exception, char *message, char *m_error,
                                  char *name, unsigned int line)
{
  if(!last_XCEPT)
    {
      last_XCEPT=exception;
      last_message=message;
      last_m_error=m_error;
      filename=name;
      fileline=line;
    }

  for(; top; xcept_pop())
    if( (top->header==exception) || (top->header==fatal_XCEPT))
      return 1;
  return 0;
}

/*------------------------------------------------------------------------*/

void *xcept_jmp_buf(void)
{
  return top?top->context_buffer:NULL;
}

/*------------------------------------------------------------------------*/

char *xcept_format(void)
{
  char message[1024];

  if(!last_message)
    last_message="error";
  if(!last_m_error)
    if(errno!=0)
      last_m_error=strerror(errno);
    else
      last_m_error=(char *) last_XCEPT;

  sprintf(message, "\033[1;5;30m%s\033[m: %s (%s:%d)",
                   last_message, last_m_error, filename, fileline);
  return strdup(message);
}

/*------------------------------------------------------------------------*/

void xcept_uncaught(void)
{
  char *message = xcept_format();
  fprintf(stderr, "%s\n", message);
  free(message);
  exit(1);
}

/*------------------------------------------------------------------------*/
