/*==========================================================================
//  Dcryptage du chiffrement de Vigenre
//  Projet TER, Matrise d'Informatique 1997,
//  Universit de Bordeaux 1, France,
//  Tous droits rservs.
//
//  de Maillard Luc, Montiton Frdric
//  Morreeuw Jean-Franois et Lompo Issa
//
//  mailto:morreeuw@emi.u-bordeaux.fr,dbv@komicom.remcomp.fr
//  http://www.emi.u_bordeaux.fr/~morreeuw/universite/TER
//
//  INTERFACE DE SELECTION D'UNE PARTIE D'UN TEXTE
//
//========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include "cipher.h"
#include "option.h"

static char *version="04/18/1997";
static char *option[]={"-h", "--help",
                       "-v", "--version",
                       "-f", "--infile",
                       "-o", "--output",
                       "-k", "--clean", NULL};
static int choice[]={1, 1, 2, 2, 4, 4, 5, 5, 6, 6};
static char *usage[]={"[-v] [-h] [-f FILENAME] [-o FILENAME] BEGIN LENGTH", NULL,
                      "Extract a part of a file",
                      "",
                      "  -f --infile   choose the file (default is STDIN)",
                      "  -o --output   save the result as a file (default is STDOUT)",
                      "  -k --clean    remove the accents from the text",
                      "  -h --help     display this help",
                      "  -v --version  output version information", NULL};

/*========================================================================*/

void
main(int argc, char **argv)
{
  char *name = argv[0];
  char *file_in = NULL;
  char *file_out = NULL;
  long begin = 0;
  long length = -1;
  short noaccent = 0;

  if(argc==1)
    opt_xusage(name, usage);
  while(*(++argv))
    switch(opt_argument(*argv, option, choice))
      {
      case 1:
        opt_usage(name, usage);
        exit(0);
        break;
      case 2:
        opt_version(name, version);
        exit(0);
        break;
      case 4:
        if(*(++argv))
          file_in = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 5:
        if(*(++argv))
          file_out = *argv;
        else
          opt_xusage(name, usage);
        break;
      case 6:
        noaccent = 1;
        break;
      default:
        if(argv[1])
          {
            begin = atoi(*argv);
            begin = (begin<0)?0:begin;
            length = atoi(*(++argv));
          }
        else
          opt_xusage(name, usage);
        break;
      }

  {
    long i;
    char *data = NULL;
    long oldlength = opt_load(file_in, &data);
    if(begin>=oldlength)
      exit(0);

    if(begin+length>oldlength || length<0)
      length = oldlength-begin;

    if(noaccent==1)
      for(i=0; i<length; i++)
        switch(data[begin+i])
          {
          case '\300':
          case '\301':
          case '\302':
          case '\303':
          case '\304':
	    data[begin+i]='A';
	    break;
          case '\340': /* grave */
          case '\341': /* acute */
          case '\342': /* circ */
          case '\343': /* tilde */
          case '\344': /* uml */
          case '\345': /* ??? */
            data[begin+i]='a';
            break;
          case '\307':
	    data[begin+i]='C';
	    break;
          case '\347': /* cedil ? */
            data[begin+i]='c';
            break;
          case '\310':
          case '\311':
          case '\312':
          case '\313':
	    data[begin+i]='E';
	    break;
          case '\350': /* grave */
          case '\351': /* acute */
          case '\352': /* circ */
          case '\353': /* uml */
            data[begin+i]='e';
            break;
          case '\314':
          case '\315':
          case '\316':
          case '\317':
	    data[begin+i]='I';
	    break;
          case '\354':
          case '\355':
          case '\356':
          case '\357':
            data[begin+i]='i';
            break;
          case '\322':
          case '\323':
          case '\324':
          case '\325':
          case '\326':
	    data[begin+i]='O';
	    break;
          case '\362':
          case '\363':
          case '\364':
          case '\365':
          case '\366':
            data[begin+i]='o';
            break;
          case '\331':
          case '\332':
          case '\333':
          case '\334':
	    data[begin+i]='U';
	    break;
          case '\371':
          case '\372':
          case '\373':
          case '\374':
            data[begin+i]='u';
            break;
          case '\320':
	    data[begin+i]='D';
	    break;
          case '\360':
            data[begin+i]='d';
            break;
          case '\321':
	    data[begin+i]='N';
	    break;
          case '\361':
            data[begin+i]='n';
            break;
          case '\336':
	    data[begin+i]='T';
	    break;
          case '\376':
            data[begin+i]='t';
            break;
          case '\337':
            data[begin+i]='s';
            break;
          case '\335':
	    data[begin+i]='Y';
	    break;
          case '\375':
          case '\377':
            data[begin+i]='y';
            break;
          default:
            break;
          }

    opt_save(file_out, &data[begin], length);
    free(data);
  }

  exit(0);
}

/*========================================================================*/
